//========================================================
//HI-MD Renderer Application - CORE CODE
//
//Written by Mark Collier.
//http://www.marcnetsystem.co.uk/
//
//This program is freeware and may be distributed in part or 
//in whole as long as a the author is credited clearly some way in 
//both the source code and in some visible way when the program 
//is running (eg:in a help screen or version info screen ).

//This notice must also appear in any source file that contains
//any part of the code in this file. 

//I (the author) take no responsability for the loss, transfer, corruption
//duplication, modification, or addition of legal or illegal data in any way. 
//
//Thanks for reading

//TO BUILD THIS YOU WILL NEED MICROSOFT DIRECTX SDK. 
//DOWNLOAD IT FROM THEM BEFORE EVEN ATTEMPTING TO BUILD THIS!

//MarC
//=========================================================

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include <strmif.h>
#include <uuids.h>
#include <vfw.h>
#include <dsound.h>
#include <control.h>
#include <evcode.h> 
#include "internal.h"

//==================Macros================================
#define RELEASE(X) if(X != NULL){X->Release();X=NULL;}
#define WriteLogCR(text) WriteLog(text,TRUE)

//=====================Types================================

typedef struct _WAVSTRUCT
{
	char id[4];
	DWORD dwLen;
	char wave_id[4];
}WAVSTRUCT, *LPWAVSTRUCT;

typedef struct _WAVCHUNK
{
	char chunkid[4];
	DWORD dwChunkDataLen;

}WAVCHUNK, *LPWAVCHUNK;

typedef struct _WAVFORMAT
{
	WORD wFormatTag;
	WORD wChannels;
	DWORD dwSampleRate;
	DWORD dwBufferEst;
	WORD wBlockAlign;
	WORD wBitsPerSample;

}WAVFORMAT, *LPWAVFORMAT;

typedef struct _CUTINFO
{
	LPBYTE lpbtCutFrom;
	DWORD dwCutLen;

}CUTINFO, *LPCUTINFO;

//=================Globals=================================
static HANDLE hLogFile = INVALID_HANDLE_VALUE;

//====================Logging functions==========================
static void WriteLog(LPSTR text,BOOL bCREnd)
{
	DWORD dwWrit;

	if(hLogFile == INVALID_HANDLE_VALUE)
	{
		printf(text);
		if(bCREnd)
		{
			printf("\n");
		}
		return;
	}

	WriteFile(hLogFile,text,strlen(text),&dwWrit,NULL);
	printf(text);
	if(bCREnd)
	{
		WriteFile(hLogFile,"\r\n",2,&dwWrit,NULL);
		printf("\n");
	}
}

static void WriteLogSignedNum(LPSTR text, int iValue,BOOL bCREnd)
{
	LPSTR buffer = (LPSTR)LocalAlloc(LPTR,strlen(text) + 500);

	if(buffer != NULL)
	{
		sprintf(buffer,"%s [%d] ",text,iValue);
		WriteLog(buffer,bCREnd);
	}

	LocalFree(buffer);
}

static void WriteLogNum(LPSTR text,DWORD dwValue,BOOL bCREnd)
{
	LPSTR buffer = (LPSTR)LocalAlloc(LPTR,strlen(text) + 500);

	if(buffer != NULL)
	{
		sprintf(buffer,"%s [0x%x=%u] ",text,dwValue,dwValue);
		WriteLog(buffer,bCREnd);
	}

	LocalFree(buffer);
}

static void WriteLogNumLongLong(LPSTR text,LONGLONG llValue,BOOL bCREnd)
{
	LPSTR buffer = (LPSTR)LocalAlloc(LPTR,strlen(text) + 500);

	if(buffer != NULL)
	{
		sprintf(buffer,"%s - [0x%I64x = %I64u]",text,llValue,llValue);
		WriteLog(buffer,bCREnd);
	}

	LocalFree(buffer);
}

static void WriteLogNumDouble(LPSTR text,double dValue,BOOL bCREnd)
{
	LPSTR buffer = (LPSTR)LocalAlloc(LPTR,strlen(text) + 500);

	if(buffer != NULL)
	{
		sprintf(buffer,"%s - [%g]",text,dValue);
		WriteLog(buffer,bCREnd);
	}

	LocalFree(buffer);
}

static void CreateLogFile(LPSTR name)
{
	char buffer[MAX_PATH];
	
	sprintf(buffer,"%s.log",name);

	hLogFile = CreateFile(buffer,GENERIC_WRITE,FILE_SHARE_READ,NULL,CREATE_ALWAYS,0,NULL);
}


static void LogSampleValues(LPSTR text, LPDWORD ldpwptr,DWORD dwNumSmps)
{
	DWORD i,x;
	signed short val;

	WriteLogCR(text);

	for(i=0; i < dwNumSmps; i++)
	{
		x = *ldpwptr;

		WriteLogNum("WindowPos:",i,FALSE);
		val = x & 0xffff;
		WriteLogSignedNum("Channel 1: ",val,FALSE);
		val = (x >> 16) & 0xffff;
		WriteLogSignedNum("Channel 2: ",val,TRUE);

		ldpwptr++;
	}
}


	

///=============================DirectShow utility functions=================================
static LONG EnumerateInterfacesOnObject(IUnknown *pUnk)
{
    HKEY k;
    LONG result = RegOpenKeyEx(HKEY_CLASSES_ROOT, "Interface", 0, KEY_READ, &k);
    if (result != NOERROR) return result;
    DWORD cSubKeys;
    DWORD cbMaxSubKeyLen;
	CHAR *pName = NULL;
	WCHAR *wstrName = NULL;
	char pDefault[255];
	DWORD cbDefault = 255;

    result = RegQueryInfoKey(k, NULL, NULL, NULL, &cSubKeys, &cbMaxSubKeyLen,
                             NULL, NULL, NULL, NULL, NULL, NULL);
	pName = (LPSTR)LocalAlloc(LPTR,cbMaxSubKeyLen + 1);
	wstrName = (LPWSTR)LocalAlloc(LPTR,(1+cbMaxSubKeyLen) * sizeof(WCHAR));

    if (result != NOERROR) return result;
    
	DWORD cbName = cbMaxSubKeyLen + 1;
        
    for (DWORD index = 0; index < cSubKeys; ++index)
    {

        cbName = cbMaxSubKeyLen + 1;
        RegEnumKeyEx(k, index, pName, &cbName, NULL, NULL, NULL, NULL);
        // Convert that into an IID
        IID iid;
		MultiByteToWideChar(CP_ACP,0,pName,-1,wstrName,cbMaxSubKeyLen);
        if (FAILED(CLSIDFromString((LPWSTR)wstrName, &iid)))
            continue;

        // Open that key, to look up the name
        HKEY k2;
        result = RegOpenKeyEx(k, pName, 0, KEY_READ, &k2);
        if (result != NOERROR) continue;
        cbDefault = 255;
        RegQueryValueEx(k2, "", NULL, NULL, (BYTE *)pDefault, &cbDefault);
        RegCloseKey(k2);
        // Query our object for it.
        IUnknown *pUnused;
        if (SUCCEEDED(pUnk->QueryInterface(iid, (void **)&pUnused)))
        {
            printf("  {IID_%s}\n", pDefault);
            pUnused->Release();
        }
    }
    LocalFree(pName);
	LocalFree(wstrName);
    RegCloseKey(k);

	return result;
}


static int DecodeRet(long lret)
{
	switch(lret)
	{
		case S_OK:
			WriteLogCR("Success !");
			return 0;
			break;
		case REGDB_E_CLASSNOTREG:
			WriteLogCR("Error: Class not registered");
			return -1;
			break;
		case CLASS_E_NOAGGREGATION :
			WriteLogCR("Error: Class not aggragatable ");
			return -2;
		case E_NOINTERFACE:
			WriteLogCR("Error: Class does not support interface ");
			return -3;
		case E_FAIL :
			WriteLogCR("Error: Failure ");
			return -4;
			break;
		case E_POINTER:
			WriteLogCR("Error: NULL pointer ");
			return -5;
		case E_INVALIDARG:
			WriteLogCR("Error: Invalid argument");
			return -6;
		case E_NOTIMPL :
			WriteLogCR("Error: not implemented");
			return -7;
		default:
			WriteLogNum("Error: Unknown error ",lret,TRUE);
			return -8;
	}

	return -8;
}

static IPin * GetPin(IBaseFilter * basefilter,BOOL bInput,AM_MEDIA_TYPE * lpamtype)
{
	IPin * ipinout = NULL;
	ULONG ulx;
	PIN_INFO pininfo;
	IEnumMediaTypes * ienummt = NULL;
	AM_MEDIA_TYPE  * amt;
	IPin * tmp;
	HRESULT hr;
	IEnumPins * enumpins = NULL;

	if( (basefilter->EnumPins(&enumpins) != S_OK) || (enumpins == NULL) )
	{
		return NULL;
	}
	
	enumpins->Reset();
	ipinout = NULL;
	while(enumpins->Next(1,&ipinout,&ulx) == S_OK)
	{
		if(ipinout == NULL) 
		{
			return NULL;
		}
		
		if(ipinout->QueryPinInfo(&pininfo) != S_OK)
		{
			return NULL;
		}

		hr = ipinout->ConnectedTo(&tmp);
		if( (hr != S_OK) && (pininfo.dir == (bInput ? PINDIR_INPUT : PINDIR_OUTPUT)) )
		{
			if((ipinout->EnumMediaTypes(&ienummt) == S_OK) && (ienummt != NULL))
			{
				if( (ienummt->Reset() == S_OK) && (ienummt->Next(1,&amt,&ulx) == S_OK))
				{
					if(lpamtype != NULL)
					{
						memcpy(lpamtype,amt,sizeof(AM_MEDIA_TYPE));
					}
					break;
				}
			
			}
		}
		
	}

	return ipinout;
}


//Direct show wav - > oma entry
static int RenderOMAFileToWav(LPSTR lpstrSrc,LPSTR lpstrDest,DWORD * lpdwLenInSecs,LPSTR errbuffer)
{
	
	long lret = 0;
	DWORD dwRet = 0;
	HRESULT hr;
	LONG levent,lparam1,lparam2;
	BOOL bExit = FALSE;
	BOOL bCheckPos = FALSE;
	WCHAR lpwstrWideChar[MAX_PATH];

	///-----Graph objects----
	IMediaControl * imedia = NULL;
	IMediaSeeking * iseek = NULL;
	IMediaPosition * ipos = NULL;
	IGraphBuilder  * igraph = NULL;
	LONGLONG lllen,llref,llstop,lltmp,lllastpos;
	REFTIME rttmp,rtref,rtlen;
	IMediaEvent * ievent = NULL;
	GUID timeformat = TIME_FORMAT_MEDIA_TIME;
	DWORD dwCap = 0;
	//----omgsrc objects----
	IUnknown * omgsrc_unknown = NULL;
	IBaseFilter * omgsrc_base= NULL;
	IPin * omgsrc_ipinout = NULL;
	AM_MEDIA_TYPE omgsrc_pinamt;
	IMediaSample * omgsrc_medsmp = NULL;
	//------omg transform---
	IUnknown * omgtrns_unknown = NULL;
	IBaseFilter * omgtrns_base= NULL;
	IPin * omgtrns_ipinin = NULL;
	IPin * omgtrns_ipinout = NULL;
	AM_MEDIA_TYPE amt_ipinout;
	//-------smart tee------------
	IUnknown * smarttee_unknown = NULL;
	IBaseFilter * smarttee_base = NULL;
	IPin * smarttee_pinin = NULL;
	IPin * smarttee_pinout = NULL;
	IPin * smarttee_pinprev = NULL;
	//------direct sound---------
	IUnknown * dsound_unknown = NULL;
	IBaseFilter * dsound_base= NULL;
	IReferenceClock * dsound_refclock = NULL;
	IPin * dsound_ipinin = NULL;
	//------sony wav writer---------
	IUnknown * sonywav_unknown = NULL;
	IBaseFilter * sonywav_base= NULL;
	IPin * sonywav_ipinin = NULL;
	IFileSinkFilter * sonywav_filesink = NULL;
	
	DWORD dwBlockSize = GetSetting(SETTING_BLOCKSIZE);
	DWORD dwSkipSize = GetSetting(SETTING_SKIPSIZE);

	CLSID clsidwavw = {0x6d6533f6, 0x5968, 0x4fb7, {0x8c, 0x00,  0x90, 0x5e,  0x71,0x57, 0x3d, 0xc4}};
	CLSID clsidomg =  {0x98660581, 0xc9a8, 0x4c92, {0xb4, 0x80,  0xf2, 0x7d , 0xe3,0xc3, 0xaa, 0xb4}};

	
	WriteLogNum("Specified Skip Size: ",GetSetting(SETTING_SKIPSIZE),TRUE);
	WriteLogNum("Specified Block Size: ",GetSetting(SETTING_BLOCKSIZE),TRUE);
	WriteLogNum("Specified differnece allowed: ",GetSetting(SETTING_ALLOWEDDIFF),TRUE);


	if(dwRet == 0)
	{
		WriteLog("Create filter graph --",FALSE);
		if(DecodeRet(CoCreateInstance(CLSID_FilterGraph,NULL,CLSCTX_INPROC_SERVER ,IID_IGraphBuilder,(void**)&igraph)) != 0)
		{
			dwRet = 1;
			sprintf(errbuffer,"Failed to create filter graph ");
		}
		else if(DecodeRet(igraph->QueryInterface(IID_IMediaControl,(void**)&imedia)) != 0)
		{
			dwRet = 1;
			sprintf(errbuffer,"Failed to query filter graph for media control");
		}
		else if(DecodeRet(igraph->QueryInterface(IID_IMediaPosition ,(void**)&ipos)) != 0)
		{
			dwRet = 1;
			sprintf(errbuffer,"Failed to query filter graph for media pos");
		}
		else if(DecodeRet(igraph->QueryInterface(IID_IMediaEvent ,(void**)&ievent)) != 0)
		{
			dwRet = 1;
			sprintf(errbuffer,"Failed to query filter graph for media event");
		}	
	}

	if(hLogFile != INVALID_HANDLE_VALUE)
	{
		igraph->SetLogFile((DWORD)hLogFile);
	}
	

	//Get IBaseFilter interface from default direct sound device
	if(dwRet == 0)
	{
		WriteLog("Get direct sound filters --",FALSE);
		if(DecodeRet(CoCreateInstance(CLSID_DSoundRender,NULL,CLSCTX_INPROC_SERVER ,IID_IUnknown,(void**)&dsound_unknown)) != 0)
		{
			dwRet = 2;
			sprintf(errbuffer,"Failed to create directsound renderer");
		}
		else if(DecodeRet(dsound_unknown->QueryInterface(IID_IBaseFilter,(void**)&dsound_base)) != 0)
		{
			dwRet = 2;
			sprintf(errbuffer,"Failed to get directsound base");
		}
		else if(DecodeRet(dsound_base->QueryInterface(IID_IReferenceClock,(void**)&dsound_refclock)) != 0)
		{
			dwRet = 3;
			sprintf(errbuffer,"Failed to get directsound clock");
		}
		
	}

	//add direct sound to graph
	if(dwRet == 0)
	{
		WriteLog("add direct sound to graph ",FALSE);
		if (DecodeRet(igraph->AddFilter(dsound_base,L"dsound")) != 0)
		{
			dwRet = 4;
			sprintf(errbuffer,"Failed to Add directsound to filter graph");
		}
	}

	//get input pin from sound 
	if(dwRet == 0)
	{
		WriteLog("get input pin from dsound ",FALSE);
		dsound_ipinin = GetPin(dsound_base,TRUE,NULL);
		if(dsound_ipinin == NULL)
		{
			dwRet = 5;
			sprintf(errbuffer,"Failed to get input pin from directsound");
		}
	}

	//Get IBaseFilter interface from OMG Source 
	if(dwRet == 0)
	{
		WriteLog("Get omgsrc_base",FALSE);
		MultiByteToWideChar(CP_ACP,0,lpstrSrc,-1,lpwstrWideChar,(MAX_PATH) -1);
		if(DecodeRet(igraph->AddSourceFilter(lpwstrWideChar,L"Src",&omgsrc_base)) != 0)
		{
			dwRet = 5;
			sprintf(errbuffer,"Failed to create source filter. Do you have SonicStage 2.1 installed?");
		}
		else if(DecodeRet(omgsrc_base->QueryInterface(IID_IUnknown,(void **)&omgsrc_unknown)) != 0)
		{
			dwRet = 5;
			sprintf(errbuffer,"Failed to query source filter");
		}
	}
		
	//get output pin from OMG Source
	if(dwRet == 0)
	{
		WriteLog("Get omgsrc_base output pin ",FALSE);
		omgsrc_ipinout = GetPin(omgsrc_base,FALSE,&omgsrc_pinamt);
		if(omgsrc_ipinout == NULL)
		{
			WriteLogCR("Failed");
			dwRet = 6;
			sprintf(errbuffer,"Failed to get output pin on source");
		}
		else 
		{
			WriteLogCR("Success ");
		}
	}
		
	//create OMG TRANSFORM
	if(dwRet == 0)
	{
		WriteLog("Create OMG TRANSFORM ",FALSE);
		
		if(DecodeRet(CoCreateInstance(clsidomg,NULL,CLSCTX_INPROC_SERVER ,IID_IUnknown,(void**)&omgtrns_unknown)) != 0)
		{
			dwRet = 7;
			sprintf(errbuffer,"Failed to create transform");
		}
		else if(DecodeRet(omgtrns_unknown->QueryInterface(IID_IBaseFilter,(void**)&omgtrns_base)) != 0)
		{
			dwRet = 7;
			sprintf(errbuffer,"Failed to get transform base");
		}
	}

	//add OMG TRNASSFORM to graph
	if(dwRet == 0)
	{
		WriteLog("Add OMG TRANSFORM to graph ",FALSE);
		if(DecodeRet(igraph->AddFilter(omgtrns_base,L"Trns")) != 0)
		{
			dwRet = 8;
			sprintf(errbuffer,"Failed to add transform to filter graph");
		}
	}


	//get input pin from omg trans
	if(dwRet == 0)
	{
		WriteLog("get input in from omg trans ",FALSE);
		if(DecodeRet(omgtrns_base->FindPin(L"Input",&omgtrns_ipinin)) != 0)
		{
			dwRet = 9;
			sprintf(errbuffer,"Failed to get input pin on transform");
		}
	}

	//get output pin from omg trans
	if(dwRet == 0)
	{
		WriteLog("get output in from omg trans ",FALSE);
		omgtrns_ipinout = GetPin(omgtrns_base,FALSE,&amt_ipinout);
		if(omgtrns_ipinout == NULL)
		{
			dwRet = 10;
			WriteLogCR(" - Error ");
			sprintf(errbuffer,"Failed to get output pin on transform");
		}
		else
		{
			WriteLogCR(" - Success! ");
		}
	}

	//Get IBaseFilter interface from sony wav writer
	if(dwRet == 0)
	{
		WriteLog("Get sony wav writer ",FALSE);
		if(DecodeRet(CoCreateInstance(clsidwavw,NULL,CLSCTX_INPROC_SERVER ,IID_IUnknown,(void**)&sonywav_unknown)) != 0)
		{
			dwRet = 11;
			sprintf(errbuffer,"Failed to create wav writer. Do you have SonicStage 2.1 installed");
		}
		else if(DecodeRet(sonywav_unknown->QueryInterface(IID_IBaseFilter,(void **)&sonywav_base)) != 0)
		{
			dwRet = 11;
			sprintf(errbuffer,"Failed to query wav writer");
		}
		else if(DecodeRet(sonywav_unknown->QueryInterface(IID_IFileSinkFilter,(void **)&sonywav_filesink)) != 0)
		{
			dwRet = 11;
			sprintf(errbuffer,"Failed to get file sink on wav writer");
		}
	}

	//add sony wav writer to graph
	if(dwRet == 0)
	{
		WriteLog("Add sony wav writer to graph ",FALSE);
		if(DecodeRet(igraph->AddFilter(sonywav_base,L"writer")) != 0)
		{
			dwRet = 12;
			sprintf(errbuffer,"Failed to add wav writer to filter graph");
		}
	}

	//set up sony wav writer output file
	if(dwRet == 0)
	{
		MultiByteToWideChar(CP_ACP,0,lpstrDest,-1,lpwstrWideChar,255);

		WriteLog("Delete output File: ",FALSE);
		WriteLog(lpstrDest,FALSE);

		if(DeleteFile(lpstrDest))
		{
			WriteLogCR(" -- Success!");
		}
		else
		{
			WriteLogCR(" -- FAILED");
		}
		
		WriteLog("Set up sony wav writer ",FALSE);
		if(DecodeRet(sonywav_filesink->SetFileName(lpwstrWideChar,NULL)) != 0)
		{
			dwRet = 13;
			sprintf(errbuffer,"Failed to set wav writer output file");
		}		
	}

	//get input pin from sony wav writer
	if(dwRet == 0)
	{
		WriteLog("get input in from sony wav writer",FALSE);
		if(DecodeRet(sonywav_base->FindPin(L"Input",&sonywav_ipinin)) != 0)
		{
			dwRet = 14;
			sprintf(errbuffer,"Failed to get input pin on wav writer");
		}
	}
	

	//get IBaseFilter interface from smarttee
	if(dwRet == 0)
	{
		WriteLog("Get smarttee interfaces ",FALSE);
		if(DecodeRet(CoCreateInstance(CLSID_SmartTee,NULL,CLSCTX_INPROC_SERVER ,IID_IUnknown,(void**)&smarttee_unknown)) != 0)
		{
			dwRet = 15;
			sprintf(errbuffer,"Failed to create smarttee");
		}
		else if(DecodeRet(smarttee_unknown->QueryInterface(IID_IBaseFilter,(void **)&smarttee_base)) != 0)
		{
			dwRet = 15;
			sprintf(errbuffer,"Failed to get smarttee base");
		}	
	}

	//add smarttee to graph
	if(dwRet == 0)
	{
		WriteLog("Add smarttee to graph ",FALSE);
		if(DecodeRet(igraph->AddFilter(smarttee_base,L"smarttee")) != 0)
		{
			dwRet = 16;
			sprintf(errbuffer,"Failed add smarttee to filter graph");
		}
	}

	//get smarttee pins
	if(dwRet == 0)
	{
		WriteLog("Get Smarttee pins ",FALSE);
		if(DecodeRet(smarttee_base->FindPin(L"Input",&smarttee_pinin)) != 0)
		{
			dwRet = 17;
			sprintf(errbuffer,"Failed get smarttee input pin");
		}
		else if(DecodeRet(smarttee_base->FindPin(L"Capture",&smarttee_pinout)) != 0)
		{
			dwRet = 17;
			sprintf(errbuffer,"Failed get smarttee output pin");
		}
		else if(DecodeRet(smarttee_base->FindPin(L"Preview",&smarttee_pinprev)) != 0)
		{
			dwRet = 17;
			sprintf(errbuffer,"Failed get smarttee preview pin");
		}
	}
	
	//attempt src->trns connection
	if(dwRet == 0)
	{
		WriteLog("Attempt connection between omgsrc and omgtrans ",FALSE);

		if(DecodeRet(igraph->Connect(omgsrc_ipinout,omgtrns_ipinin)) != 0)
		{
 			dwRet = 18;
			sprintf(errbuffer,"Failed to connect source to transform. You need SonicStage 2.1 to do this.");
		}
	}

	//attempt trns->smarttee connection
	if(dwRet == 0)
	{
		WriteLog("Attempt connection between  omgtrans and smarttee ", FALSE);
		if(DecodeRet(igraph->Connect(omgtrns_ipinout,smarttee_pinin)) != 0)
		{
 			dwRet = 19;
			sprintf(errbuffer,"Failed to connect transform to smarttee. You need SonicStage 2.1 to do this.");
		}
	
	}

	//attempt smarttee->wav writer connection
	if(dwRet == 0)
	{
		WriteLog("Attempt connection between  smarttee and sony wav writer ",FALSE);
		if(DecodeRet(igraph->Connect(smarttee_pinout,sonywav_ipinin)) != 0)
		{
 			dwRet = 20;
			sprintf(errbuffer,"Failed to smarttee to wav writer");
		}
	
	}

	//connect smarttee and direct sound
	if(dwRet == 0)
	{
		WriteLog("Attempt connection between  smarttee and directsound ",FALSE);
		if(DecodeRet(igraph->Connect(smarttee_pinprev,dsound_ipinin)) != 0)
		{
 			dwRet = 21;
			sprintf(errbuffer,"Failed to smarttee to directsound");
		}
	}

	//set up graph 
	if(dwRet == 0)
	{
		WriteLog("Setting up graph ",FALSE);
		
		if(DecodeRet(igraph->QueryInterface(IID_IMediaSeeking ,(void**)&iseek)) != 0)
		{
			dwRet = 22;
			sprintf(errbuffer,"Failed to get mediaseeking interface");
		}
		else if(DecodeRet(iseek->SetTimeFormat(&timeformat)) != 0)
		{
			dwRet = 22;
			sprintf(errbuffer,"Failed to set time format");
		}
		else if (DecodeRet(ievent->CancelDefaultHandling(EC_COMPLETE)) != 0)
		{
			dwRet = 23;
			sprintf(errbuffer,"Failed to cancel events");
		}
		else if(DecodeRet(iseek->GetDuration(&lllen)) != 0)
		{
			dwRet = 23;
			sprintf(errbuffer,"Failed to get duration of input (1)");
		}
		else if(DecodeRet(ipos->get_Duration(&rtlen)) != 0)
		{
			dwRet = 23;
			sprintf(errbuffer,"Failed to get duration of input (2)");
		}

		*lpdwLenInSecs = (DWORD)rtlen;
		dwCap = 0xffffffff;
		hr = iseek->GetCapabilities(&dwCap);
		iseek->GetPreroll(&llref);
		WriteLogNum("Capabilities:",dwCap,FALSE);
		WriteLogNumLongLong(" preroll:" ,llref,TRUE);	
		
	}


	//Run filter
	if(dwRet == 0)
	{
		WriteLogNum("Run. omgsrc= ",(DWORD)omgsrc_base,FALSE);
		WriteLogNum(" omgtrns=", (DWORD)omgtrns_base,FALSE);
		WriteLogNum("smarttee=", (DWORD)smarttee_base,FALSE);
		WriteLogNum("sonywav=", (DWORD)sonywav_base,FALSE);
		WriteLogNum("sound=",(DWORD)dsound_base,TRUE);
			
		rtref=  0;
		llref = 0;
		llstop = dwBlockSize * 10000000;
		hr = iseek->SetPositions(&llref,AM_SEEKING_AbsolutePositioning | AM_SEEKING_Segment ,
				                &llstop,AM_SEEKING_AbsolutePositioning | AM_SEEKING_Segment  );
		if(hr != S_OK)
		{
			WriteLogNum("Error setting end ",hr,TRUE);
			ipos->put_StopTime(rtref + dwBlockSize);
		}


		imedia->Run();
		lllastpos = -1;
		do
		{
			levent = 0;
			lparam2 = 0;
			lparam1 = 0;
			ievent->GetEvent(&levent,&lparam1,&lparam2,INFINITE);
			switch(levent)
			{
				case EC_PAUSED:		
					
					break;
				case EC_COMPLETE:
					hr = iseek->GetCurrentPosition(&lltmp);
					if(hr != S_OK)
					{
						WriteLogNum("Error getting fine pos ",hr,TRUE);
						ipos->get_CurrentPosition(&rttmp);
						lltmp = (LONGLONG)rttmp * 10000000;
					}
					

					if((hr == S_OK) && (lltmp >= lllen) && (lllastpos == lltmp))
					  
					{
						WriteLogNumLongLong("At End. Pos: ",lltmp,TRUE);
						WriteLogNumLongLong("At End. lastpos: ",lllastpos,TRUE);
						WriteLogNumLongLong("At End. len: ",lllen,TRUE);
						bExit = TRUE;
					}
					else
					{	
						//pause and read current position
						imedia->Pause();
						hr = iseek->GetCurrentPosition(&lltmp);
						WriteLogNumLongLong("Stopped at ",lltmp,TRUE);

						//set resume position
						rtref += dwBlockSize - 1;
						llref +=  (dwBlockSize * 10000000) - dwSkipSize; //(dwBlockSize * 1000000);
						llstop = llref + ((dwBlockSize * 10000000) + 10000000);//(dwBlockSize * 1000000));
						lllastpos = lltmp;
			
						if(llstop >= lllen)
						{	//we're near the end... Lets just run to the end. without a stop position
																			
							hr = iseek->SetPositions(&llref,AM_SEEKING_AbsolutePositioning | AM_SEEKING_NoFlush ,
													 &lllen,AM_SEEKING_AbsolutePositioning | AM_SEEKING_NoFlush);
						}
						else
						{
							hr = iseek->SetPositions(&llref,AM_SEEKING_AbsolutePositioning | AM_SEEKING_NoFlush ,
													 &llstop,AM_SEEKING_AbsolutePositioning | AM_SEEKING_NoFlush);
						}


						if(hr != S_OK)
						{
							WriteLogNum("error seeking ",hr,TRUE);
							ipos->put_CurrentPosition(rtref - 1);
							ipos->put_StopTime(rtref + dwBlockSize);
						}

						//resume filter graph
						imedia->Run();						
					}
					
					break;
			}
							
			ipos->get_CurrentPosition(&rttmp);
			WriteLogNum("Pos: ",(DWORD)rttmp,FALSE);
			WriteLogNum("Len: ",(DWORD)rtlen,FALSE);
			WriteLogNumLongLong("Pos (secs): ",lltmp,FALSE);
			WriteLogNumLongLong("Len (secs): ",lllen,FALSE);
			WriteLogNum("Event: ",levent,TRUE);
		}
		while(bExit == FALSE);
	}

	
	//Release everything
	///-----Graph objects----
	RELEASE(imedia);
	RELEASE(iseek);
	RELEASE(ipos);
	RELEASE(igraph);
	RELEASE(ievent);
	//----omgsrc objects----
	RELEASE(omgsrc_unknown);
	RELEASE(omgsrc_base);
	RELEASE(omgsrc_ipinout);
	RELEASE(omgsrc_medsmp);
	//------omg transform---
	RELEASE(omgtrns_unknown);
	RELEASE(omgtrns_base);
	RELEASE(omgtrns_ipinin);
	RELEASE(omgtrns_ipinout);
	//-------smart tee------------
	RELEASE(smarttee_unknown);
	RELEASE(smarttee_base);
	RELEASE(smarttee_pinin);
	RELEASE(smarttee_pinout);
	RELEASE(smarttee_pinprev );
	//------direct sound---------
	RELEASE(dsound_unknown);
	RELEASE(dsound_base);
	RELEASE(dsound_refclock);
	RELEASE(dsound_ipinin);
	//------sony wav writer---------
	RELEASE(sonywav_unknown);
	RELEASE(sonywav_base);
	RELEASE(sonywav_ipinin);
	RELEASE(sonywav_filesink);

	return 0;
}

//==================================WAV fixing functions===============================

static BOOL __inline CompareSamples(DWORD dwSample1,DWORD dwSample2,DWORD dwAllowedDiff)
{
	DWORD a,b,dwDif;
	
	if(dwSample1 != dwSample2 )
	{	
		//not equal... but very small differences are allowed
		a = dwSample1 & 0xffff;
		b = dwSample2 & 0xffff;
		dwDif = (a > b) ? a - b : b - a;
		if((dwDif > dwAllowedDiff))
		{
			return FALSE;
		}

		a = dwSample1 >> 16;
		b = dwSample2 >> 16;
		dwDif = (a > b) ? a - b : b - a;
		if((dwDif > dwAllowedDiff))
		{
			return FALSE;
		}
	}

	return TRUE;
}

static DWORD __inline FindStartOfRepeat(LPBYTE lpbtWavData, DWORD dwSectionOff,
										DWORD dwSearchOff, DWORD  dwRptSrchOff)
{
	LPDWORD lpdwdSrch = (LPDWORD) &lpbtWavData[dwSearchOff + dwSectionOff]; 
	LPDWORD lpdwdRpt = (LPDWORD) &lpbtWavData[dwRptSrchOff + dwSectionOff];
	DWORD dwSub = 0;
	DWORD dwADif = GetSetting(SETTING_ALLOWEDDIFF);

	while(CompareSamples(*lpdwdSrch,*lpdwdRpt,dwADif))
	{
		lpdwdRpt--;
		lpdwdSrch--;
		dwSub += 4;
	}

	return dwSub;
	
}

static DWORD __inline FindLenOfRepeat(LPBYTE lpbtWavData,DWORD dwWavLen,  DWORD dwSectionOff, DWORD dwMaxRptSzInBytes,
									  DWORD dwSearchOff, DWORD  dwRptSrchOff, DWORD dwAvgRptLen)
{
	DWORD dwRptLen;
	LPDWORD lpdwdSrch = (LPDWORD) &lpbtWavData[dwSearchOff + dwSectionOff + 4]; 
	LPDWORD lpdwdRpt = (LPDWORD) &lpbtWavData[dwRptSrchOff + dwSectionOff + 4];
	DWORD dwAlwdDif = GetSetting(SETTING_ALLOWEDDIFF);

	//now scan for end of repeat
	dwRptLen = 4;
	while( (dwRptLen < dwMaxRptSzInBytes) && 
		   ((dwSectionOff + dwRptLen + dwRptSrchOff) < dwWavLen)
		 )
	{
		if(!CompareSamples(*lpdwdSrch,*lpdwdRpt,dwAlwdDif))
		{			
			return dwRptLen;
		}
		
		lpdwdSrch++;
		lpdwdRpt++;
		dwRptLen += 4;
	}	

	if(GetSetting(SETTING_NOIGNORELONGRPTS) == TRUE)
	{
		WriteLogNum("Repeat is longer than maximum. Using Average",dwAvgRptLen,TRUE);
		return dwAvgRptLen;
	}

	WriteLogNum("Repeat is longer than maximum. Ignoring",dwMaxRptSzInBytes,TRUE);
	return 0;
}


static BOOL __inline FindRepeatInSection(LPBYTE lpbtWavData, DWORD dwWavLen, DWORD dwSectOff,DWORD dwSectionLen,
										 DWORD dwMinRptSzInBytes,DWORD dwMaxRptSzInBytes,DWORD dwAvgRptLen,
										 DWORD * lpdwRptOff, DWORD * lpdwRptLen)
{
	DWORD dwRptLen ;
	DWORD dwSearchOff;
	DWORD dwRptSrchOff;
	DWORD dwSmp,dwCmpSmp;
	DWORD dwSub;
	DWORD dwAmount = dwSectionLen / 2;
	DWORD i;	
	DWORD dwAlwdDif = GetSetting(SETTING_ALLOWEDDIFF);

	
	for(i=0; i < 4;i++)
	{
		WriteLogNum("Using search method ",i,TRUE);
		switch(i)
		{
			case 0:
				dwAmount = dwSectionLen / 2;
				dwSearchOff = dwAmount;
				break;
			case 1:
				dwAmount = dwSectionLen / 2;
				dwSearchOff = dwAmount;
				break;
			case 2:
				dwSearchOff = 8;
				break;
			case 3:
				dwSectionLen *= 2;
				dwSearchOff = 0;
				break;
		}
			
		while( (dwSearchOff >= 8) && (dwSearchOff < (dwSectionLen - dwMinRptSzInBytes)) && 
			   ((dwSearchOff+ dwSectOff) < dwWavLen)
			)
		{
			dwSearchOff -= (dwSearchOff % 4);
			dwSmp = *(DWORD *)&lpbtWavData[dwSectOff + dwSearchOff];

			//search for repeat of sample
			dwRptSrchOff = dwSearchOff + 4;
			while( (dwRptSrchOff < dwSectionLen) && 
				   ((dwRptSrchOff + dwSectOff) < dwWavLen)
				 )
			{
				dwCmpSmp = *(DWORD *)&lpbtWavData[dwSectOff + dwRptSrchOff];
				if(CompareSamples(dwSmp,dwCmpSmp,dwAlwdDif))
				{	//match... but is it long enough
					
					dwRptLen = FindLenOfRepeat(lpbtWavData,dwWavLen, 
											   dwSectOff,dwMaxRptSzInBytes,
											   dwSearchOff,dwRptSrchOff,dwAvgRptLen);

					if(dwRptLen == 0)
					{	//scanned from dwSectOff+4 to dwMaxRptSzInBytes and nothing found
						dwRptSrchOff += dwMaxRptSzInBytes;
					}
					else if((dwRptLen >= dwMinRptSzInBytes)   && (dwRptLen < dwMaxRptSzInBytes))
					{
						//find start of repeat and return TRUE
						dwSub = FindStartOfRepeat(lpbtWavData,dwSectOff,dwSearchOff,dwRptSrchOff);
						*lpdwRptOff = dwSearchOff - dwSub;
						*lpdwRptLen = dwRptSrchOff  - dwSearchOff;

						return TRUE;
					}
					else
					{	//repeat block is too small... try again
						dwRptSrchOff += dwRptLen + 4;
					}

				}
				else
				{
					dwRptSrchOff += 4;
				}
			}
			
			switch(i)
			{
				case 0:
					dwAmount /= 2;
					dwSearchOff = dwAmount;
					break;
				case 1:
					dwAmount /= 2;
					dwSearchOff = (dwSectionLen - dwAmount);
					break;
				case 2:
					dwSearchOff += dwMinRptSzInBytes / 2;
					break;
				case 3:
					dwSearchOff += dwMinRptSzInBytes / 4;
					break;
			}
		}
	}

	//not found
	return FALSE;
}

static LPCUTINFO ScanWave(LPBYTE lpbtWavData,DWORD dwWavDataLen,WAVFORMAT * lpwavformat,
						   double dSectionDistInSecs,double dSectionSizeInSecs,
						   double dMinRepeatSizeInSecs,double dMaxRepeatSizeInSecs)
{
	DWORD dwCurSection;
	DWORD dwSectionPosInBytes,dwSectionSizeInBytes,dwMinRptSzInBytes,dwMaxRptSzInBytes;
	DWORD dwChnsBytesPerSample;
	DWORD dwRptOff,dwRptLen;
	DWORD dwMaxCuts,dwCutIndex;
	double x,y, dist,totrptoff,totrptlen;
	LPCUTINFO lpcutinfo;
	DWORD bUseAvging = GetSetting(SETTING_USEAVGING);

	//init
	totrptoff = 0;
	dwCurSection = 1;
	dwSectionPosInBytes = 0;
	totrptlen = 0;
	dwChnsBytesPerSample = (lpwavformat->wChannels * (lpwavformat->wBitsPerSample / 8));
	
	//calculate Section size, in bytes
	x = (dSectionSizeInSecs * lpwavformat->dwSampleRate * dwChnsBytesPerSample);
	dwSectionSizeInBytes = (DWORD)x;
	
	//calculate Section distance, in bytes
	dist= dSectionDistInSecs * lpwavformat->dwSampleRate * dwChnsBytesPerSample;
	dwSectionPosInBytes = (DWORD)dist;
	dwSectionPosInBytes -= dwSectionSizeInBytes / 2;
	 		
	//calculate minimum allowed size of repeat data, in bytes
	x = dMinRepeatSizeInSecs * lpwavformat->dwSampleRate * dwChnsBytesPerSample;
	dwMinRptSzInBytes = (DWORD)x;

	//calculate maximum allowed size of repeat data, in bytes
	x = dMaxRepeatSizeInSecs * lpwavformat->dwSampleRate * dwChnsBytesPerSample;
	dwMaxRptSzInBytes = (DWORD)x;

	//calculate maximum number of cut informations that need storing
	x = dwWavDataLen / dist;
	dwMaxCuts = (DWORD)(x + 10);
	dwCutIndex = 0;
	lpcutinfo = (LPCUTINFO)LocalAlloc(LPTR,dwMaxCuts * sizeof(CUTINFO));
	memset(lpcutinfo ,0 ,dwMaxCuts * sizeof(CUTINFO));
	
	while(dwSectionPosInBytes < dwWavDataLen)
	{
		if((dwSectionPosInBytes + dwSectionSizeInBytes) > dwWavDataLen)
		{
			dwSectionPosInBytes = dwWavDataLen - dwSectionSizeInBytes;
		}
		
		dwSectionPosInBytes -= (dwSectionPosInBytes % 4);
		x = ((dwSectionPosInBytes + dwSectionSizeInBytes) / dwChnsBytesPerSample) / lpwavformat->dwSampleRate;
		y = (dwSectionPosInBytes / dwChnsBytesPerSample) / lpwavformat->dwSampleRate;
		
		WriteLogCR("-------");
		WriteLogNum("SECTION:",dwCurSection,FALSE);
		WriteLogNum("Pos:",dwSectionPosInBytes,FALSE);
		WriteLogNum("Len:",dwWavDataLen, FALSE);
		WriteLogNum("Current section:",dwSectionPosInBytes,FALSE);
		WriteLogNum("Current section size:",dwSectionSizeInBytes,FALSE);
		WriteLogNumDouble("Avg rpt offset:",totrptoff / dwCurSection,FALSE);
		WriteLogNumDouble("Avg rpt len:",totrptlen / dwCurSection,FALSE);
		WriteLogNumDouble("Section from ",y,FALSE);
		WriteLogNumDouble("to ",x,TRUE);

		if( (dwSectionPosInBytes + dwSectionSizeInBytes) > dwWavDataLen)
		{
			dwSectionSizeInBytes = (dwSectionPosInBytes + dwSectionSizeInBytes) - dwWavDataLen;
			if((dwMaxRptSzInBytes + dwSectionSizeInBytes) > dwWavDataLen)
			{
				dwMaxRptSzInBytes = (dwSectionPosInBytes + dwSectionSizeInBytes) - dwWavDataLen;
			}
		}
		
		//is there repeated audio in this section?
		x = totrptlen / dwCurSection;
		if(FindRepeatInSection(lpbtWavData ,dwWavDataLen, dwSectionPosInBytes,dwSectionSizeInBytes,
								dwMinRptSzInBytes,dwMaxRptSzInBytes,(DWORD)x,
								&dwRptOff,&dwRptLen) == TRUE)
		{	
			//found a repeat in the specified section
			x = dwRptLen;
			totrptlen += dwRptLen;
			x /=  dwChnsBytesPerSample * lpwavformat->dwSampleRate;
			WriteLogNum("   Found repeat -> Offset",dwRptOff,FALSE);
			WriteLogNum(" Length:",dwRptLen,FALSE);
			WriteLogNumDouble(" len in secs:",x,TRUE);


			//set up cut info
			lpcutinfo[dwCutIndex].lpbtCutFrom = lpbtWavData + dwSectionPosInBytes + dwRptOff;
			lpcutinfo[dwCutIndex].dwCutLen = dwRptLen;
			dwCutIndex++;

			if(!bUseAvging)
			{
				dwSectionPosInBytes += dwRptOff - dwRptLen;

				//keep stats up to date even though we dont use them here
				dwCurSection++;
				totrptoff += dwRptOff;
			}
			else if( (int)dwRptOff > 0)
			{
				totrptoff += dwRptOff;
				dwSectionPosInBytes += (DWORD)(totrptoff / dwCurSection) ;
				dwCurSection++;
			}
			else
			{	//negative offset
				totrptoff += (totrptoff / dwCurSection) ; //keep same average....
				dwSectionPosInBytes += dwRptOff; //remeber that dwRptOff is negative
				dwCurSection++;
			}		
		}
		else
		{	//no
			WriteLogCR("    NOT Found repeat ");			
		}

		dwSectionPosInBytes += (DWORD)dist;
							
	}

	return lpcutinfo;
}


static DWORD CalculateCutInfoSize(LPCUTINFO lpcutinfo)
{
	DWORD dwRet = 0;
	while(lpcutinfo->lpbtCutFrom != NULL)
	{
		dwRet += lpcutinfo->dwCutLen;
		lpcutinfo++;
	}

	return dwRet;
}


static void SmoothSampleData(LPDWORD lpdwSamplesStart,DWORD dwNumSmps,
							 LPDWORD lpdwSmoothStart,DWORD dwNumSmpsToSmooth)
{
	DWORD dwTotDifa,dwTotDifb,dwAvgDifa,dwAvgDifb,i;
	DWORD dwNumnotsmooth;
	double calc;
	BOOL bSmoothed = FALSE;
	signed short a0,b0,a1,b1,a2,b2,da1,da2,db1,db2;
	struct { DWORD start;DWORD end;}notsmooth[1024];
	
	//init
	dwNumnotsmooth = 0;
	dwTotDifa = 0;
	dwTotDifb = 0;
	memset(notsmooth,0,sizeof(notsmooth));

	//first get average difference
	for(i=1;i<= dwNumSmps;i++)
	{
		a0 = (signed short)(lpdwSamplesStart[i-1] & 0xffff);
		b0 = (signed short)(lpdwSamplesStart[i-1] >> 16);
		a1 = (signed short)(lpdwSamplesStart[i] & 0xffff);
		b1 = (signed short)(lpdwSamplesStart[i] >> 16);
		
		dwTotDifa += (a0 > a1) ? a0 - a1 : a1 - a0;
		dwTotDifb += (b0 > b1) ? b0 - b1 : b1 - b0;
	}
	
	dwAvgDifa = dwTotDifa / dwNumSmps;
	dwAvgDifb = dwTotDifb / dwNumSmps;

	
	WriteLogNum("Average difference ",dwAvgDifa,FALSE);
	WriteLogNum(" and ",dwAvgDifb,FALSE);
	WriteLogNum("over samples ",dwNumSmps,TRUE);
		

	i=1;
	LogSampleValues("Old Averaging Window",lpdwSmoothStart,dwNumSmpsToSmooth);
	while(i< dwNumSmpsToSmooth)
	{
		bSmoothed = FALSE;
		a0 = (signed short)(lpdwSmoothStart[i-1] & 0xffff);
		b0 = (signed short)(lpdwSmoothStart[i-1] >> 16);
		a1 = (signed short)(lpdwSmoothStart[i] & 0xffff);
		b1 = (signed short)(lpdwSmoothStart[i] >> 16);
		a2 = (signed short)(lpdwSmoothStart[i+1] & 0xffff);
		b2 = (signed short)(lpdwSmoothStart[i+1] >> 16);

		da1 = (a0 > a1) ? a0-a1 : a1-a0;
		db1 = (b0 > b1) ? b0-b1 : b1-b0;
		da2 = (a1 > a2) ? a1-a2 : a2-a1;
		db2 = (b1 > b2) ? b1-b2 : b2-b1;

		if( (da1 > (int)dwAvgDifa) && 
			(da2 > (int)dwAvgDifa) &&
			( ((a0 > a1) && (a2 > a1)) || ((a0 < a1) && (a2 < a1)) )
		  )
		{
			WriteLogNum("Smoothed channel 1 at avg pos",i ,FALSE);
			WriteLogSignedNum("old val",a1,FALSE);
			
			bSmoothed = TRUE;
			calc = (double)(a0 + a2) / 2;
			a1 = (signed short)calc;

			WriteLogSignedNum("new val",a1,TRUE);
		}

		if( (db1 > (int)dwAvgDifb) && 
			(db2 > (int)dwAvgDifb) &&
			( ((b0 > b1) && (b2 > b1)) ||  ((b0 < b1) && (b2 < b1)) )
		  )
		{			
			WriteLogNum("Smoothed channel 2 at avg pos",i  ,FALSE);
			WriteLogSignedNum("old val",b1,FALSE);
							
			bSmoothed = TRUE;
			calc = (double)(b0 + b2) / 2;
			b1 = (signed short)calc;

			WriteLogSignedNum("new val",b1,TRUE);
		}

		if(bSmoothed)
		{
			lpdwSmoothStart[i] = (b1 << 16) | (a1 & 0xffff);
			i += 2;
		}
		else
		{
			i++;
		}
	}

	LogSampleValues("New Averaging Window",lpdwSmoothStart,dwNumSmpsToSmooth);

}



//repair entry function
static BOOL RepairWav(LPSTR lpstrWavFile,LPSTR outfile,DWORD dwLenInSecs,
					  double  dSectionDistInSecs,double dSectionSizeInSecs,
					  double dMinRepeatSizeInSecs,double dMaxRepeatSizeInSecs,LPSTR errbuf)
{
	HANDLE hFile,hFileOut,hmap;
	LPBYTE lpbtWavFile;
	LPBYTE lpbtWavData;
	LPBYTE lpbtTmp;
	WAVSTRUCT * lpwavstruct;
	WAVCHUNK * lpwavchunk;
	WAVFORMAT * lpwavformat;
	DWORD dwFileSize;
	DWORD dwWavDataLen;
	LPCUTINFO lpcutinfo;
	char buffer[255];
	DWORD dwWrit;
	DWORD dwValue;
	DWORD i;
	LPBYTE lpbtFrom;
	DWORD dwSmooAnlWnd = GetSetting(SETTING_SMOOTH_ANALYSIS_WINDOW);
	DWORD dwSmooAvgWnd = GetSetting(SETTING_SMOOTH_AVGING_WINDOW);
	DWORD dwSmooAnlWndBytes,dwSmooAvgWndBytes;
	DWORD adwSmoothingBuffer[1024];

	
	hFile = CreateFile(lpstrWavFile,GENERIC_READ | GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,0);
	if(hFile == INVALID_HANDLE_VALUE)
	{
		WriteLog("RepairWav => Failed to open source wave ",FALSE);
		WriteLogCR(lpstrWavFile);
		sprintf(errbuf,"Failed to open temp wave file");
		return FALSE;
	}

	hFileOut = CreateFile(outfile,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0);
	if(hFileOut == INVALID_HANDLE_VALUE)
	{
		WriteLog("RepairWav => Failed to open dest wave ",FALSE);
		sprintf(errbuf,"Failed to open dest wave file");
		WriteLogCR(outfile);
		CloseHandle(hFile);
		return FALSE;
	}

	dwFileSize = GetFileSize(hFile,NULL);

	hmap = CreateFileMapping(hFile,NULL,PAGE_READONLY,0,0,NULL);
	if(hmap == NULL)
	{
		WriteLog("RepairWav => Failed to create mapped file ",FALSE);
		sprintf(errbuf,"Failed to create mapped file");
		WriteLogCR(lpstrWavFile);
		CloseHandle(hFile);
		CloseHandle(hFileOut);
		return FALSE;
	}

	lpbtWavFile = (LPBYTE)MapViewOfFile(hmap,FILE_MAP_READ,0,0,0);
	if(lpbtWavFile == NULL)
	{
		WriteLog("RepairWav => Failed map view of file ",FALSE);
		sprintf(errbuf,"Failed to map view of temp file");
		WriteLogCR(lpstrWavFile);
		CloseHandle(hmap);
		CloseHandle(hFile);
		CloseHandle(hFileOut);
		return FALSE;
	}

	//parse wav file header
	lpwavstruct = (WAVSTRUCT *)lpbtWavFile;
	lpwavchunk = (WAVCHUNK *)&lpbtWavFile[sizeof(WAVSTRUCT)];
	while(lpwavchunk != NULL)
	{
		lpbtTmp = (LPBYTE)lpwavchunk;
		if( (lpwavchunk->chunkid[0] == 'f') && (lpwavchunk->chunkid[1] == 'm') &&
			(lpwavchunk->chunkid[2] == 't') )
		{	//fmt chunk.. copy useful bits off
			lpwavformat = (WAVFORMAT *)(lpbtTmp + sizeof(WAVCHUNK));
			if(lpwavformat->wFormatTag != 0x0001)
			{
				lpwavformat = NULL;
				WriteLogCR("Wav Format not PCM ");
				sprintf(errbuf,"tmp WAV is not PCM or is corrupt");
				break;
			}

			lpbtTmp += sizeof(WAVFORMAT) +  sizeof(WAVCHUNK);
			lpwavchunk = (WAVCHUNK *)lpbtTmp;
		}
		else if( (lpwavchunk->chunkid[0] == 'd') && (lpwavchunk->chunkid[1] == 'a') &&
			     (lpwavchunk->chunkid[2] == 't') )
		{	//data chunk
			lpbtWavData = (LPBYTE)lpwavchunk;
			lpbtWavData+=2;
			dwWavDataLen = lpwavchunk->dwChunkDataLen;
			if(dwWavDataLen == 0)
			{//guess

				dwWavDataLen += (DWORD)lpbtWavData;
				dwWavDataLen -= (DWORD)lpbtWavFile;
				dwWavDataLen = dwFileSize - dwWavDataLen;
				dwWavDataLen -= sizeof(WAVCHUNK);
			}


			lpbtWavData += sizeof(WAVCHUNK);
			break;
		}
		else
		{	//invalid
			sprintf(errbuf,"invalid field in temp wav");
			break;
		}
		
	}
	
	//check all required wave components are there....
	if( (lpwavchunk == NULL)  || (lpbtWavData == NULL) )
	{
		UnmapViewOfFile(lpbtWavFile);
		lpbtWavFile = NULL;
		CloseHandle(hmap);
		CloseHandle(hFileOut);
		CloseHandle(hFile);
		WriteLogCR("Missing data in wave header");
		strcat(errbuf,". Missing or corrupt temp file!");
		return FALSE;
	}

	//scan wave
	lpcutinfo = NULL;
	lpcutinfo = ScanWave(lpbtWavData,dwWavDataLen,lpwavformat,
						 dSectionDistInSecs,dSectionSizeInSecs,
						 dMinRepeatSizeInSecs,dMaxRepeatSizeInSecs);

	//------Create New Wav----------
	//write RIFF header
	sprintf(buffer,"RIFF");	
	dwValue = lpwavstruct->dwLen - CalculateCutInfoSize(lpcutinfo);
	WriteFile(hFileOut,buffer,4,&dwWrit,NULL);	//
	WriteFile(hFileOut,&dwValue,4,&dwWrit,NULL);
	sprintf(buffer,"WAVE");
	WriteFile(hFileOut,buffer,4,&dwWrit,NULL);

	//write format chunk
	sprintf(buffer,"fmt ");
	WriteFile(hFileOut,buffer,4,&dwWrit,NULL);	//chunk id
	dwValue = 16;
	WriteFile(hFileOut,&dwValue,4,&dwWrit,NULL);	//chunk data size
	WriteFile(hFileOut,&lpwavformat->wFormatTag,2,&dwWrit,NULL);	//compression code
	WriteFile(hFileOut,&lpwavformat->wChannels,2,&dwWrit,NULL);	//num channels
	WriteFile(hFileOut,&lpwavformat->dwSampleRate,4,&dwWrit,NULL);	//sample rate
	WriteFile(hFileOut,&lpwavformat->dwBufferEst,4,&dwWrit,NULL); //average bytes per second
	WriteFile(hFileOut,&lpwavformat->wBlockAlign,2,&dwWrit,NULL); //block align
	WriteFile(hFileOut,&lpwavformat->wBitsPerSample,2,&dwWrit,NULL); //bits per sample
	dwValue = 0;
	
	//write data chunk
	sprintf(buffer,"data");
	WriteFile(hFileOut,buffer,4,&dwWrit,NULL);	//chunk id
	dwValue = dwWavDataLen - CalculateCutInfoSize(lpcutinfo);
	WriteFile(hFileOut,&dwValue,4,&dwWrit,NULL);	//new wave data len
	
	i=0;
	lpbtFrom = lpbtWavData;
	dwSmooAnlWndBytes = dwSmooAnlWnd * (lpwavformat->wBitsPerSample / 8) * lpwavformat->wChannels;
	dwSmooAvgWndBytes = dwSmooAvgWnd * (lpwavformat->wBitsPerSample / 8) * lpwavformat->wChannels;
	while(lpbtFrom < (lpbtWavData + dwWavDataLen))
	{
		//copy to beginning of current cut start position
		if(lpcutinfo[i].lpbtCutFrom != NULL)
		{
			if((GetSetting(SETTING_ENABLESMOOOTHING) == TRUE) && (dwSmooAnlWnd > 0))
			{
				//copy end of current block to smoothing buffer ... 
				memcpy(adwSmoothingBuffer,
					   lpcutinfo[i].lpbtCutFrom - (dwSmooAnlWndBytes/2),
					   dwSmooAnlWndBytes / 2);
				
				//copy start of next block to smoothing buffer. Copy an additional sample for the smoother
				memcpy(adwSmoothingBuffer + (dwSmooAnlWnd / 2),
					   lpcutinfo[i].lpbtCutFrom + lpcutinfo[i].dwCutLen, 
					   (dwSmooAnlWndBytes / 2) + 4);

				WriteLogCR("================");
				WriteLogNum("Smoothing block ",i+1,TRUE);
				//LogSampleValues("Before smooth",adwSmoothingBuffer,dwSmooAnlWnd);
				
				//smooth data
				SmoothSampleData(adwSmoothingBuffer,dwSmooAnlWnd,
							     adwSmoothingBuffer + ((dwSmooAnlWnd / 2) - (dwSmooAvgWnd / 2)),
								 dwSmooAvgWnd);

				//LogSampleValues("After smooth",adwSmoothingBuffer,dwSmooAnlWnd);

				//write from lpbtfrom (either start of data or [start of current block + 16] )
				//to start of cut data - (dwSmooWndBytes / 2)
				WriteFile(hFileOut,lpbtFrom,lpcutinfo[i].lpbtCutFrom - lpbtFrom - (dwSmooAnlWndBytes / 2),
						 &dwWrit,NULL);

				//write content of smooth buffer
				WriteFile(hFileOut,adwSmoothingBuffer,dwSmooAnlWndBytes,&dwWrit,NULL);

				lpbtFrom = lpcutinfo[i].lpbtCutFrom + lpcutinfo[i].dwCutLen;
				
				//The smoothing buffer included the start of the block... so skip appropriate amount...
				lpbtFrom += (dwSmooAnlWndBytes / 2);

			}
			else
			{
				WriteFile(hFileOut,lpbtFrom,lpcutinfo[i].lpbtCutFrom - lpbtFrom ,&dwWrit,NULL);
				lpbtFrom = lpcutinfo[i].lpbtCutFrom + lpcutinfo[i].dwCutLen ;
			}

			i++;

		}
		else
		{	//finished... just copy from current pos to end of data
			if(((lpbtWavData + dwWavDataLen) - lpbtFrom) > 0)
			{
				WriteFile(hFileOut,lpbtFrom,(lpbtWavData + dwWavDataLen) - lpbtFrom,&dwWrit,NULL);
			}
			
			break;
		}
	}
	
	//clean up
	if(lpcutinfo != NULL)
	{
		LocalFree(lpcutinfo);
	}
 	UnmapViewOfFile(lpbtWavFile);
	lpbtWavFile = NULL;
	CloseHandle(hmap);
	CloseHandle(hFile);
	CloseHandle(hFileOut);


	if(GetSetting(SETTING_KEEPTMPFILE) == FALSE)
	{
		DeleteFile(lpstrWavFile);
	}

	return TRUE;
}


void InitCore(LPSTR logfile)
{
	WriteLogCR("Init");
	CoInitialize(NULL);
	if(GetSetting(SETTING_DEBUG) == TRUE)
	{
		CreateLogFile(logfile);
	}
}

void ChangeLogFile(LPSTR logfile)
{
	CloseHandle(hLogFile);
	hLogFile = INVALID_HANDLE_VALUE;
	if(GetSetting(SETTING_DEBUG) == TRUE)
	{
		CreateLogFile(logfile);
	}
}


void DeinitCore()
{
	WriteLogCR("Deinit'ed");
	
	CoUninitialize();
	if(hLogFile != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hLogFile);
		hLogFile = INVALID_HANDLE_VALUE;
	}
}


int DoCoreConversion(LPSTR lpstrInFile, LPSTR lpstrTempFile, LPSTR lpstrDestFile,LPSTR errbuf)
{
	DWORD dwLenInSecs = 0;
	DWORD dwBlockSize,dwSkipSize;
	int iRet;
	
	//render OMA to temporarry wav
	iRet = RenderOMAFileToWav(lpstrInFile,lpstrTempFile,&dwLenInSecs,errbuf);
	//iRet = 0;
	//remove repeats from wav
	if(iRet == 0)
	{
		WriteLog("Processing File:",FALSE);
		WriteLogCR(lpstrInFile);
		dwBlockSize = GetSetting(SETTING_BLOCKSIZE);
		dwSkipSize = GetSetting(SETTING_SKIPSIZE);
		if(!RepairWav(lpstrTempFile,lpstrDestFile,dwLenInSecs,dwBlockSize, (double)dwBlockSize / 10, 0.1,(double)dwSkipSize / 2500000,errbuf))
		{
			return -1;
		}
	}

	return iRet;
}